

function [varargout] = getMixedSecDerivativeViaDifferenceQuotient(fun,x0,y0,fSize)
%Numerical differentiation of the function DyDxfun(x0,y0)
%takes the numerical approximation 
% Dy_iDx_j=(f(x0+eps/2*ei,y0+eps/2*ej)-f(x0-eps/2*ei,y0+eps/2*ej)-f(x0+eps/2*ei,y0-eps/2*ej)+f(x0-eps/2*ei,y0-eps/2*ej)/eps^2

myEps=10^-6;

    nrOutputs=length(fSize);
    xSize=size(x0,2);
    ySize=size(y0,2);
    
    
for n=1:nrOutputs
    if size(fSize{n},2)==0
        varargout{n}=zeros([xSize,ySize]);
    else
        varargout{n}=zeros([fSize{n},xSize,ySize]);       	
    end
    totalNumberElementsFun{n}=prod(fSize{n});
    indices{n}=1:1:totalNumberElementsFun{n};
end

f1=cell(1,nrOutputs);
f2=cell(1,nrOutputs);
f3=cell(1,nrOutputs);
f4=cell(1,nrOutputs);

for i=1:ySize
    for j=1:xSize
        x1=x0;
        x2=x0;
        y1=y0;
        y2=y0;
        x1(j)=x1(j)+myEps/2;
        x2(j)=x2(j)-myEps/2;
        y1(i)=y1(i)+myEps/2;
        y2(i)=y2(i)-myEps/2;
        
        [f1{:}]=fun(x1,y1);
        [f2{:}]=fun(x2,y1);
        [f3{:}]=fun(x1,y2);
        [f4{:}]=fun(x2,y2);        
        
        
        for n=1:nrOutputs
            if any(fSize{n}==0)            
            else
                varargout{n}(indices{n}+(j-1)*totalNumberElementsFun{n}+(i-1)*totalNumberElementsFun{n}*xSize)=(f1{n}-f2{n}-f3{n}+f4{n})/(myEps^2);
            end

        end
        for n=1:nrOutputs
            varargout{n}(isnan(varargout{n}))=0;
        end
    end
end


end