function [] = checkProblem( problem,checkDerivatives,checkSecDerivatives )
%checkProblem(p,checkDeriv,checkSecDeriv) checks all functions of the
%problem p. If a output has the wrong size or an inconsistent value a
%warning is thrown. It may happen, that in a first run only a few warnings
%are thrown. If they are corrected more mistakes may be found.If
%checkDeriv is 1, derivatives are checked. if checkSecDeriv is 1, seccond
%derivatives are checked.


needstest=1;
if isa(problem,'DCProblemAsGSIP')
   disp('testing design')
   checkDesign(problem.design,checkDerivatives,checkSecDerivatives)
   disp('testing container')
   checkContainer(problem.container,checkDerivatives,checkSecDerivatives)
   needstest=0;
end

if isa(problem,'MBProblemAsGSIP')
    n=problem.getNrIndexSets;
    for i=1:n
        disp(['testing design ',num2str(i)]);
        checkDesign(problem.design{i},checkDerivatives,checkSecDerivatives);
    end
    disp('testing container')
    checkContainer(problem.container,checkDerivatives,checkSecDerivatives);
    needstest=0;
end

if isa(problem,'PortfolioAsGSIP')
    disp('testing UcSet')
    UcSet=problem.UcSet;
    checkUcSet(UcSet,checkDerivatives,checkSecDerivatives);
    needstest=0;    
end

if needstest
works=1;
%UpperLevel no information needed
bgetNrUppLevVars=checkOneFunction(problem,'getNrUppLevVars',{[1;1]});
bgetNrUppLevNonlinIneqs=checkOneFunction(problem,'getNrUppLevNonlinIneqs',{[1;1]});
bgetNrUppLevNonlinEqs=checkOneFunction(problem,'getNrUppLevNonlinEqs',{[1;1]});
bgetNrSemiInfConstrs=checkOneFunction(problem,'getNrSemiInfConstrs',{[1;1]}); 
%LowerLevel no information needed
bgetNrIndexSets=checkOneFunction(problem,'getNrIndexSets',{[1;1]});

bgetNrLowLevVars=0;
if bgetNrIndexSets
    nrOfIndexset=problem.getNrIndexSets;
    for i=1:nrOfIndexset
        bgetNrLowLevVars=checkOneFunction(problem,'getNrLowLevVars',{[1;1]},i);
    end
end

if bgetNrUppLevVars
    m=problem.getNrUppLevVars;
   
    bgetBoundsOnUppLevVars=checkOneFunction(problem,'getBoundsOnUppLevVars',{[1,m],[1,m]});
    bgetLinEqsOnUppLevVars=checkOneFunction(problem,'getLinEqsOnUppLevVars',{[inf,m],[inf,1]});
    if bgetLinEqsOnUppLevVars
        [C,~]=problem.getLinEqsOnUppLevVars;
        bgetLinEqsOnUppLevVars=checkOneFunction(problem,'getLinEqsOnUppLevVars',{[size(C,1),m],[size(C,1),1]});
    end
    bgetLinIneqsOnUppLevVars=checkOneFunction(problem,'getLinIneqsOnUppLevVars',{[inf,m],[inf,1]});
    if bgetLinIneqsOnUppLevVars
        [A,~]=problem.getLinIneqsOnUppLevVars;
        bgetLinIneqsOnUppLevVars=checkOneFunction(problem,'getLinIneqsOnUppLevVars',{[size(A,1),m],[size(A,1),1]});
    end

    
    bgetinitValsOfUppLevVars=checkOneFunction(problem,'getInitValsUppLevVars',{[1;m]});
    
    works=min([works,bgetinitValsOfUppLevVars]);
    
    if bgetinitValsOfUppLevVars
        x=problem.getInitValsUppLevVars;
        bsetInitValsUppLevVars=checkOneFunction(problem,'setInitValsUppLevVars',{},x);

        bevalObjFun=checkOneFunction(problem,'evalObjFun',{[1;1]},x);
        if bevalObjFun && checkDerivatives
             bevalObjFun=checkOneFunctionDerivative(problem,'gradObjFun',1,'evalObjFun',1,{[]},x,1);
             bevalObjFun=min(bevalObjFun,checkOneFunctionDerivative(problem,'DxObjFun',0,'evalObjFun',1,{[]},x,1));  
        end
        if bevalObjFun && checkSecDerivatives
            bevalObjFun=checkOneFunctionSecDerivative(problem,'DxDxObjFun','evalObjFun',1,{[]},x,1); 
        end
        
        
        if bgetNrUppLevNonlinEqs&&bgetNrUppLevNonlinIneqs
            n1=problem.getNrUppLevNonlinEqs;
            n2=problem.getNrUppLevNonlinIneqs;
            
            bevalUppLevNonlinIneqConstrs=checkOneFunction(problem,'evalUppLevNonlinIneqConstrs',{[1;n2]},x);
            if bevalUppLevNonlinIneqConstrs && checkDerivatives
                bevalUppLevNonlinIneqConstrs=checkOneFunctionDerivative(problem,'DxUppLevNonlinIneqConstrs',0,'evalUppLevNonlinIneqConstrs',1,{n2},x,1);
            end
            if bevalUppLevNonlinIneqConstrs && checkSecDerivatives
                bevalUppLevNonlinIneqConstrs=checkOneFunctionSecDerivative(problem,'DxDxUppLevNonlinIneqConstrs','evalUppLevNonlinIneqConstrs',1,{n2},x,1);
            end
            bevalUppLevNonlinEqConstrs=checkOneFunction(problem,'evalUppLevNonlinEqConstrs',{[1;n1]},x);
            if bevalUppLevNonlinEqConstrs && checkDerivatives
                bevalUppLevNonlinEqConstrs=checkOneFunctionDerivative(problem,'DxUppLevNonlinEqConstrs',0,'evalUppLevNonlinEqConstrs',1,{n1},x,1);
            end
            if bevalUppLevNonlinEqConstrs && checkSecDerivatives
                bevalUppLevNonlinEqConstrs=checkOneFunctionSecDerivative(problem,'DxDxUppLevNonlinEqConstrs','evalUppLevNonlinEqConstrs',1,{n1},x,1);
            end
            works=min([works,bevalUppLevNonlinEqConstrs,bevalUppLevNonlinIneqConstrs]);
            
        end
        works=min([works,bevalObjFun,bsetInitValsUppLevVars]);
        
        
       
    
   
    
        if isa(problem,'ImplicitGSIPProblem') && bgetNrSemiInfConstrs && bgetNrLowLevVars && bgetNrIndexSets 
            works2=checkImplicitGSIPProblem(problem,x,checkDerivatives,checkSecDerivatives);
            works=min(works2,works);
        end

        if isa(problem,'ExplicitProblem')&& bgetNrIndexSets && bgetNrLowLevVars&& bgetNrSemiInfConstrs
            works2=checkExplicitProblem(problem,x);
            works=min(works2,works);
        end
        works=min([works,bgetLinIneqsOnUppLevVars,bgetLinEqsOnUppLevVars,bgetBoundsOnUppLevVars]);
    end
    
end
works=min([works,bgetNrIndexSets,bgetNrLowLevVars,bgetNrSemiInfConstrs,bgetNrUppLevNonlinEqs,bgetNrUppLevNonlinIneqs,...
   bgetNrUppLevVars]); 

bgetIndexSetOfSemiInfConstr=checkOneFunction(problem,'getIndexSetOfSemiInfConstr',{[1;1]},1);

works =min(works,bgetIndexSetOfSemiInfConstr);


if works && ~isa(problem,'MBProblemAsGSIP') && ~isa(problem,'DCProblemAsGSIP')
    disp('all methods passed the tests');
    
end


end
end

