classdef ContainerParamHyperCube < Container
    %C=ContainerParamHyperCube(n,l,p) creates an instance of the container C given by:
    %
    %       C={ y in R^n with:
    %               -yi <= 0 for i=1,...,n  
    %               yi-l <= 0 for i=n+1,...,2n
    %               yn - 4*(l-p)/l^2 * sum_i=1^n-1(yi-l/2)^2  -p}
    %
    %   Bounding box: [0,l]^n


    properties (GetAccess=protected)
        dim                                
        nrOfConstFuns                      
        BoundingBox;
        
        edgeLength;
        parabola
    end
    
    methods
        function p=ContainerParamHyperCube(dim,edgeLength,parabola)
            p.dim=dim;
            p.nrOfConstFuns=2*dim;
            p.edgeLength=edgeLength;
            assert(parabola<edgeLength);
            p.parabola=parabola;
            p.nrOfConstFuns=p.nrOfConstFuns+1;
            p.BoundingBox=[zeros(1,dim);edgeLength*ones(1,dim)];
            
        end
    end
    
    methods 
        %container functions
        function g=evalDescrFuns(obj,i,y)   
            if i<=obj.dim
                g=-y(:,i);
            elseif i>obj.dim && obj.dim*2>=i
                g=y(:,i-obj.dim)-obj.edgeLength;
                
            else
                l=obj.edgeLength;
                p=obj.parabola;
                g=y(:,end)-4*(l-p)/l^2*((y(:,1:end-1)-l/2).^2*ones(obj.dim-1,1))-p;                
            end

        end
        %derivative
        function Dy_g=DyDescrFuns(obj,i,y)  
            m=size(y,1);
            
            if i<=obj.dim
                Dy_g=zeros(m,obj.dim);
                Dy_g(:,i)=-ones(m,1);
            elseif i>obj.dim && obj.dim*2>=i
                Dy_g=zeros(m,obj.dim);
                Dy_g(:,i-obj.dim)=ones(m,1);
            else
                l=obj.edgeLength;
                p=obj.parabola;
                Dy_g=[-8*(l-p)/l^2*(y(:,1:end-1)-l/2),ones(m,1)];
            end
           
        end
    end
    
    methods
        function plotContainer(obj)
            switch obj.dim
                case 2
                   l=obj.edgeLength;
                    x1=(0:l*0.05:l)';
                    p=obj.parabola;
                    y1=4*(l-p)/l^2*(x1-l/2).^2+obj.parabola;


                    X = [0; ... %vertex
                         0; ...%vertex
                         x1;... %parabola
                        l]; %vertex
                    Y = [0; ...        %vertex
                         l; ...%vertex
                        y1;... %parabola
                        0];     %vertex

                    ph = fill(X, Y, [135/255, 206/255, 250/255]); % sky blue
                    set(ph, 'EdgeColor', 'blue');
                    hold on; 
                case 3
                    l=obj.edgeLength;
                    p=obj.parabola;
                    fill3([0,0,l,l],[0,l,l,0],[0,0,0,0],'blue', 'facealpha', 0.25);
                    hold on
                    fill3([0,0,l,l],[0,0,0,0],[0,l,l,0],'blue', 'facealpha', 0.25);
                    fill3([0,0,0,0],[0,l,l,0],[0,0,l,l],'blue', 'facealpha', 0.25);


                    fill3([0,0,l,l],[l,l,l,l],[0,l,l,0],'blue', 'facealpha', 0.25);
                    fill3([l,l,l,l],[0,l,l,0],[0,0,l,l],'blue', 'facealpha', 0.25);
                    
                        %Bestimmen der oberen Fl�che
                        x=(0:l*0.05:l);
                        y=(0:l*0.05:l);
                        [X,Y]=meshgrid(x,y);
                        Zt=4*(l-p)/l^2*((X-l/2).^2+(Y-l/2).^2)+p;
                        Z=min(l*ones(size(Zt)),Zt);
                        s=surf(X,Y,Z);
                        set(s,'FaceColor', 'blue', 'FaceAlpha', 0.25);

                    hold on;
            end
        end
    end
    
    
end
