classdef ImplicitProblemTemplate<ImplicitGSIPProblem %The class has to be renamed. The class-name has to coincide with the file-name.
    properties(GetAccess=protected) %If the properties ire fixed, they can 
                                    %be set directly here. Otherwise the 
                                    %constructor below can be used to set
                                    %the properties
        %Upper Level
        nrOfUppLevVars              %Number of upper level variables x
        nrOfSemiInfConstrs          %Number of constraints with infinitely many indices
        
        nrOfNonlinIneqsUppLev       %Has to coincide with the number of
                                    %constraints implemented in function
                                    %evalNonlinIneqUppLev
                                    
        nrOfNonlinEqsUppLev         %Has to coincide with the number of 
                                    %constraints implemented in function
                                    %evalNonlinEqUppLev      
                          
        linIneqsOnUppLevVars        %Inequalities of the form A*x <= b. 
                                    %Saved as [A,b]. If there are no linear
                                    %inequalities: [].
        
        linEqsOnUppLevVars          %Equalities of the form C*x = d. Saved
                                    %as [C,d]. If there are no linear
                                    %inequalities: [].
        
        boundsOnUppLevVars          %Bounds of the form lb <= x <= ub.
                                    %Saved as [lb;ub]. If there are no
                                    %bounds lb=-inf, ub=inf. 
        
        initValsOfUppLevVars        %Initial values for the optimization.
        
        %Problem description       
        nrOfIndexset                %Number of different indexsets
        con2LowLev                  %saves the connection of the 
                                    %semi-infinite constraint to the
                                    %coressponding index set. For every
                                    %semi-infinite constraint one entry
                                    %with the index of the indexset. If
                                    %there is only one indexset, a vector
                                    %of ones is needed.
                                       
                                        
        
        %Lower Level
        nrOfLowLevVars;             %Dimension of the indexvariables. If
                                    %there are more than 1 indexsets, then 
                                    %the different dimensions are saved in 
                                    %a vector                   
        nrOfLowLevNonlinConstrs;    %Number of non linear constraints 
                                    %describing the index-set
    end
    
    methods %Have to be filled
        
        %%%%%%%%%%%%%
        %Constructor%
        %%%%%%%%%%%%%
        
        %The name of the constructor has to be changed and coincide with 
        %the class name
        function p=ImplicitProblemTemplate() %The constructor can also have
                                             %some inputs
            
            %Descriptions of the properties can be found above or in the
            %documentation
            
            %%%%%%%%%%%%%%%%%%%%%
            % have to be filled %
            %%%%%%%%%%%%%%%%%%%%%
            p.nrOfUppLevVars =1;              
            p.nrOfSemiInfConstrs =1;          
            p.nrOfLowLevVars=1;
            
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            % can be deleted in the most cases %
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            
            %If some initial values are at hand they can be set. Otherwise
            %the next line is not needed
            p.initValsOfUppLevVars=[0];
            
            %If there is only one index-set, the following can be deleted.
            %Otherwise it has to be filled:
            p.nrOfIndexset=1;                 
            p.con2LowLev=[1];
            
            
            %if there are none of the following constraints they can be
            %deleted
            p.linIneqsOnUppLevVars = [];      
            p.linEqsOnUppLevVars = [];        
            p.boundsOnUppLevVars =[-inf;inf];
            
            
            %If there are no nonlinear constraints on the upper level vars 
            %or the function is always evaluatable the following properties 
            %can be set automatically and the two lines can be deleted.
            p.nrOfNonlinIneqsUppLev = 0;      
            p.nrOfNonlinEqsUppLev =0;
            
            %If there are no nonlinear constraints on the lower level
            %variables the following line can be deleted.                                            
            p.nrOfLowLevNonlinConstrs=0;
            
            
            %%%%%%%%%%%%%%%%%%%%%%%
            %should not be deleted%
            %%%%%%%%%%%%%%%%%%%%%%%
            
            %Sets the remaining variables automatically
            p.setDependendValuesForProperties();
                       
        end
        
        %%%%%%%%%%%%%%%%%%%% 
        %objective function%
        %%%%%%%%%%%%%%%%%%%%        
        function f=evalObjFun(obj,x)                   
            f=0;            
        end
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%
        %semi-infinite Constraints%
        %%%%%%%%%%%%%%%%%%%%%%%%%%%
        function g=evalSemiInfConstrs(obj,i,x,y)     
            g=0;            
        end
    end
    
    
    methods %The following functions can be filled, but can also be deleted if not needed.
        
        %%%%%%%%%%%%
        %UpperLevel%
        %%%%%%%%%%%%
        % Upper level nonlinear Constraints
        function k=evalUppLevNonlinIneqConstrs(obj,x)
            %g=evalUppLevNonlinIneqConstrs(x) evaluates the non linear and
            %non semi-infinite inequality constraints on the parameters x. 
            %The output g is a vector. Every entry corresponds to one 
            %constraint. If the function is not implemented, a empty vector 
            %of size 1x0 is returned.
            k=[];                
        end
        function k=evalUppLevNonlinEqConstrs(obj,x)
            %h=evalUppLevNonlinEqConstrs(x) evaluates the non linear and
            %non semi-infinite equality constraints on the parameters x. 
            %The output h is a vector. Every entry corresponds to one 
            %constraint. If the function is not implemented, a empty vector 
            %of size 1x0 is returned.
            k=[];                
        end
        
        %%%%%%%%%%%%
        %LowerLevel%
        %%%%%%%%%%%%
        %Starting Point lower level
        function y=getInitValsLowLevVars(obj,j,x)           
            y=0;            
        end
        %Nonlinear constraints on lower level variables
        function u=evalLowLevNonlinConstrs(obj,j,x,y)
            %u=evalLowLevNonlinConstrs(j,x,y) evaluates the non linear 
            %constraint of the j-th index set. The output u is a vector
            %where every entry corresponds to one constraint. If no such
            %constraints are implemented, a empty vector of size 1x0 is
            %returned.
            u=[];            
        end
        
        
        %Linear inequalities on lower level variables
        function [A,b]=getLinIneqsOnLowLevVars(obj,j,x)
            %[A,b]=getLinIneqsOnLowLevVars(j,x) returns the linear
            %inequality constraints of the j-th index set A(x)*y<=b(x).
            A=[];
            b=[];
        end
        
        %Bounds on the lower level constraints
        function [lb,ub]=getBoundsOnLowLevVars(obj,j,x)
            %[lb,ub]=getBoundsOnLowLevVars(j,x) returns a bounding box
            %of the j-th index set for a fixed point x. lb and ub are
            %rowvectors. If one variable yi is unbounded, the bounds are
            %-inf/inf.
             lb=-inf*ones(1,obj.nrOfLowLevVars);
             ub=inf*ones(1,obj.nrOfLowLevVars);
        end
        
        %%%%%%%%%%%%%%%%
        %Transformation%
        %%%%%%%%%%%%%%%%
        %transformation function 
        function y=evalTrafoFun(obj,i,x,z)
            %y=evalTrafoFun(i,x,z) evaluates the transformation function
            %for the i-th index set. 
            y=z;           
        end
        %inverse transformation function
        function z=evalInvTrafoFun(obj,i,x,y)
            %z=evalInvTrafoFun(i,x,y) returns one preimage of the
            %transformation for the i-th index set.
            z=y;            
        end

    end
    
    %If needed, derivatives can be implemented as described in the
    %documentation. If no derivatives are implemented, numerical derivatives
    %or derivatives computed by automatic differentiation can be used.
    
end

