classdef Problem1<ImplicitGSIPProblem
    % p=Problem1() creates an instance of the GSIP problem:
    %
    %       min   x1+x2
    %
    %       s.t.  -y<=0 for all y in Y(x) 
    %             -1 <= x1 <= 1
    %             -1 <= x2 <= 1
    %
    %       where:
    %
    %       Y(x)={y in R with
    %               -y <= -x1
    %               -y <= -x2 }
    
    properties(GetAccess=protected)
       
        %Upper Level
        nrOfUppLevVars = 2;
        nrOfSemiInfConstrs =1;
        
        nrOfNonlinIneqsUppLev = 0;
        nrOfNonlinEqsUppLev   =  0;           
                          
        linIneqsOnUppLevVars = [];
        linEqsOnUppLevVars = [];
        
        boundsOnUppLevVars =[-1,-1;1,1];
        
        initValsOfUppLevVars =[0,1];
        
        %Lower Level
        nrOfLowLevVars  =1                        
        nrOfLowLevNonlinConstrs =0;
        
        %Problem        
        nrOfIndexset =1
        con2LowLev =1

    end
    methods
        function p=Problem1()
            %Creates an instance of problem 1
            p.linIneqsOnUppLevVars=zeros(0,3);
            p.linEqsOnUppLevVars=zeros(0,3);
        end
    end
    methods
        %Upper-Level 
        %objective function
        function f=evalObjFun(~,x)                   
            f=x(1)+x(2);
            
        end
        function grad_f=gradObjFun(obj,x) 
            grad_f=[1;1];
        end
        
        %semi-infinite Constraints
        function g=evalSemiInfConstrs(~,~,x,y)     
            g=-y;
            
        end
        function Dx_g=DxSemiInfConstrs(~,~,~,y)      
            m=size(y,1);
            Dx_g=zeros(m,2); 
          
        end
        function Dy_g=DySemiInfConstrs(~,~,~,y)
            m=size(y,1);
            Dy_g=-ones(m,1);             
        end
               
        %LowerLevel
        %Linear inequalities
        function [A,b]=getLinIneqsOnLowLevVars(obj,k,x)
            A=[-1;-1];
            b=[-x(1);-x(2)];
        end
        
        %Starting Point
        function y=getInitValsLowLevVars(~,~,x)           
            y=x(1)^2+x(2)^2+1;
            
        end
        function Dx_y=DxStartingPointLowLev(~,~,x)    
            Dx_y=[2*x(1),2*x(2)];
        end     

        

        
        %transformation
        function y=evalTrafoFun(obj,i,x,z)
          
            if(x(1)>x(2))
                y=x(1)+z;
            else
                y=x(2)+z;
            end
        end
        function z=evalInvTrafoFun(obj,i,x,y)
            
            if x(1)>x(2)
                z=(y-x(1));
            else
                z=(y-x(2));
            end
        end
        function Dx_y=DxTrafoFun(obj,i,x,z)
            m=size(z,1);
            Dx_y=zeros(1,2,m);
            if x(1)>x(2)
                Dx_y(1,1,:)=ones(1,m);
            else
                Dx_y(1,2,:)=ones(1,m);
            end
        end
    end
    
end

