
classdef Problem7< ImplicitGSIPProblem
    % p=Problem7() creates an instance of the GSIP problem:
    %
    %       min   x2
    %
    %       s.t.  -x2-y^3 <=0 for all y in Y(x) 
    %             2x2-x1^2+10^-6 <= 0
    %
    %       where:
    %
    %       Y(x)={y in R with
    %               2x2-y^3-x1^2 <= 0
    %               y <=0}
    
        %The lower level problem is non-convex. One constraint is added. 
    properties(GetAccess=protected)
        
       
        nrOfUppLevVars=2;                         
        boundsOnUppLevVars=[-inf,-inf;inf,inf];    
        nrOfSemiInfConstrs=1;                      
        initValsOfUppLevVars=[1,0];               
        
        linEqsOnUppLevVars=[];                    
        linIneqsOnUppLevVars=[];                  
        nrOfNonlinEqsUppLev=0;               
        nrOfNonlinIneqsUppLev=1;             
        
        
        nrOfLowLevVars=1;                         
        nrOfLowLevNonlinConstrs=1;
        
        con2LowLev=1;
        nrOfIndexset=1;        

        
    end

    methods
        %objective function
        function f=evalObjFun(~,x)                    
            f=x(2);
        end
        function grad_f=gradObjFun(~,~)                   
            grad_f=[0;1];
        end
        
        %upper level
        function g=evalSemiInfConstrs(~,~,x,y)        
            g=-x(2)-y(:).^3;
            
        end
        function Dx_g=DxSemiInfConstrs(~,~,~,y)      
            m=size(y,1);
            Dx_g=[zeros(m,1),-ones(m,1)];                
        end
        function Dy_g=DySemiInfConstrs(~,~,~,y)
            Dy_g=-3*y(:).^2;                
        end
         
        
        
        function k=evalUppLevNonlinIneqConstrs(~,x)       
            k=2*x(2)-x(1)^2+1.e-6;                
        end
        function Dx_k=DxUppLevNonlinIneqConstrs(~,x)      
            Dx_k=[-2*x(1),2];                
        end

        
        % lower level
        function u=evalLowLevNonlinConstrs(~,~,x,y)        
            u=[2*x(2)-y^3-x(1)^2];
            
        end
        function Dy_u=DyLowLevNonlinConstrs(~,~,~,y)            
            Dy_u=[-3*y^2];                        
        end
        function Dx_u=DxLowLevNonlinConstrs(~,~,x,~)
            Dx_u=[-2*x(1), 2] ;
        end
        
        function y=getInitValsLowLevVars(~,~,x)         
            y=nthroot((2*x(2)-x(1)^2),3)/2;
            
        end
        function Dx_y=DxStartingPointLowLev(~,~,x)        
            Dx_y=[-x(1)/3*nthroot((2*x(2)-x(1)^2),3)^-2,1/3*nthroot(2*x(2)-x(1)^2,3)^-2];
        end
        
        function [lb,ub]=getBoundsOnLowLevVars(~,~,~)    
             lb=-inf;
             ub=0;
        end
                        
        
        %transformation
        function y=evalTrafoFun(obj,i,x,z)
            [lb,ub]=obj.getInterval(x);
            y=lb*z+ub*(1-z);
        end
        function z=evalInvTrafoFun(obj,i,x,y)
            [lb,ub]=obj.getInterval(x);
            z=(y-ub)/(lb-ub);
                        
        end
        function Dx_y=DxTrafoFun(obj,i,x,z)
            [Dx_lb,Dx_ub]=obj.DxInterval(x);
            Dx_y=zeros(1,size(x,2),size(z,1));
            o=ones(size(z));
          
            Dx_y(1,:,:)=(o*Dx_lb*z+(1-z)*(Dx_ub))';   
        end
    end
    methods(Access=private)
       
        function [lb,ub]=getInterval(~,x)                
            lb=nthroot((2*x(2)-x(1)^2),3);
             ub=0;
        end
        function [Dx_lb,Dx_ub]=DxInterval(~,x)          
            Dx_lb=[-2*x(1)/3*nthroot((2*x(2)-x(1)^2),3)^-2,2/3*nthroot(2*x(2)-x(1)^2,3)^-2];
            Dx_ub=[0,0];
        end
        
        
        
    end

    

    
end